@@ -8,11 +8,16 @@ use lemmy_api_common::{
   get_local_user_view_from_jwt,
   get_local_user_view_from_jwt_opt,
   is_admin,
-  resolve_actor_identifier,
   send_application_approved_email,
   site::*,
 };
-use lemmy_apub::fetcher::search::{search_by_apub_id, SearchableObjects};
+use lemmy_apub::{
+  fetcher::{
+    resolve_actor_identifier,
+    search::{search_by_apub_id, SearchableObjects},
+  },
+  objects::community::ApubCommunity,
+};
 use lemmy_db_schema::{
   diesel_option_overwrite,
   from_opt_str_to_opt_enum,
@@ -197,7 +202,7 @@ impl Perform for Search {
     let search_type: SearchType = from_opt_str_to_opt_enum(&data.type_).unwrap_or(SearchType::All);
     let community_id = data.community_id;
     let community_actor_id = if let Some(name) = &data.community_name {
-      resolve_actor_identifier::<Community>(name, context.pool())
+      resolve_actor_identifier::<ApubCommunity, Community>(name, context)
         .await
         .ok()
         .map(|c| c.actor_id)
@@ -416,11 +421,9 @@ impl Perform for ResolveObject {
 
     let res = search_by_apub_id(&self.q, context)
       .await
-      .map_err(LemmyError::from)
       .map_err(|e| e.with_message("couldnt_find_object"))?;
     convert_response(res, local_user_view.map(|l| l.person.id), context.pool())
       .await
-      .map_err(LemmyError::from)
       .map_err(|e| e.with_message("couldnt_find_object"))
   }
 }
@@ -505,12 +508,8 @@ impl Perform for LeaveAdmin {
     let site_view = blocking(context.pool(), SiteView::read_local).await??;
     let admins = blocking(context.pool(), PersonViewSafe::admins).await??;
 
-    let federated_instances = build_federated_instances(
-      context.pool(),
-      &context.settings().federation,
-      &context.settings().hostname,
-    )
-    .await?;
+    let federated_instances =
+      build_federated_instances(context.pool(), &context.settings()).await?;
 
     Ok(GetSiteResponse {
       site_view: Some(site_view),
@@ -565,7 +564,6 @@ impl Perform for SaveSiteConfig {
 
     // Make sure docker doesn't have :ro at the end of the volume, so its not a read-only filesystem
     let config_hjson = Settings::save_config_file(&data.config_hjson)
-      .map_err(LemmyError::from)
       .map_err(|e| e.with_message("couldnt_update_site"))?;
 
     Ok(GetSiteConfigResponse { config_hjson })
